// ****************************************************************************
// *                                                                          *
// * File : mixvibesdsp.h                                                     *
// * Header for the class : mixvibesDSPModule                                 *
// *                                                                          *
// ****************************************************************************
// * Description :                                                            * 
// *     Base interface for all MixVibes DSP plugins                          *
// *                                                                          *
// ****************************************************************************
// * CREATION DATE : 15/11/04                                                 *
// * LAST MODIFIED : 23/12/04                                                 *
// ****************************************************************************
// * VERSION : 0.99                                                           *
// * AUTHORS : eric_g                                                         *
// *           alex_p                                                         *
// ****************************************************************************
// * COPYRIGTH (C) 2004 - InVibes - ALL RIGHTS RESERVED                       *
// ****************************************************************************

#ifndef MIXVIBES_DSP_INC
#define MIXVIBES_DSP_INC

#include <windows.h>  // ** Plugins are Windows-only for now **

///////////////////////////////////////////////////////////////////////////////

#define	MIXVIBES_DSP_SDK_VERSION 1       // * Current MixVibes DSP-SDK version  *

///////////////////////////////////////////////////////////////////////////////

// --- Specific parameter type definitions ---
typedef	float MVDSP_PARAM_VALUE;         // * Parameters are 32bit IEEE floats  *
typedef unsigned int MVDSP_PARAM_TYPE;   // * Parameter types are UINT          *
typedef unsigned int MVDSP_PARAM_FLAGS;  // * Parameter flags are UINT          *

// --- Important parameter related constants ---
#define	MVDSP_MAX_NB_PARAM           13  // * Plugin may have up to MAX params  *
#define MVDSP_PARAM_DISPLAY_MAX_SIZE 50  // * Max size of param display string  *

// --- Parameter special flags ---
#define MVDSP_PARAM_NO_FLAG      0       // * Set when no special flag needed   *
#define MVDSP_PARAM_FLAG_IS_LOG  0x1     // * Logarithmic changes needed        *
#define MVDSP_PARAM_FLAG_IS_INV  0x1<<1  // * Inverted meaning (max is min)     *

// --- Parameter control types : which GUI control to display ---
#ifndef TYPEPARAMINT
#define	TYPEPARAMINT				 1   // * Fader with integer granularity    *
#define	TYPEPARAMREAL1				 2   // * Fader with 0.1 granularity        *
#define TYPEPARAMREAL2				 3   // * Fader with 0.01 granularity       *
#define TYPEPARAMREAL3				 4   // * Fader with 0.001 granularity      *
#define	TYPEPARAMBOOL				 5   // * ON/OFF button                     *
#define	TYPEPARAMTIME				 6   // * Fader (integer) + nT support (*)  *
#define	TYPEPARAMLFO				 7   // * LFO-curve selection control       *
#define	TYPEPARAMCOMBOLIST			 8   // * Combo list filled by host (*)     *
#define	TYPEPARAMCOMBOLISTTIME       9   // * Combo list filled with time mult. *
#endif

  // * Notes : nT support means user may enter time multiples in the parameter  *
  // *         edit box (example: '1T' means 'one quarte note', while 0.25T is  *
  // *         1/4 quarter note = 1/16 note)                                    *
  // *                                                                          *
  // * Notes : combo box are filled by the host. Each integer value between     *
  // *         the min and max parameter bounds is 'probed' : Host calls        *
  // *         ParamDisplay() to get the display string of each value and add   *
  // *         it to the combo box.                                             *

///////////////////////////////////////////////////////////////////////////////

// --- Plugin parameter ---
typedef struct			   
{
	MVDSP_PARAM_TYPE type;               // * Parameter type                    *
	MVDSP_PARAM_FLAGS flags;             // * Parameter flags                   *    
	const char* text;                    // * Parameter name to display         *
	MVDSP_PARAM_VALUE defVal;            // * Parameter default value           *
	MVDSP_PARAM_VALUE min;               // * Parameter min value               *
	MVDSP_PARAM_VALUE max;               // * Parameter max value               *

} mixvibesParam;

///////////////////////////////////////////////////////////////////////////////

// --- Plugin flags : tell which methods are supported or not ---
#define MVDSP_CANDO_BLOCK_PROCESS  0x1       // * Process() method is available *
#define MVDSP_CANDO_SAMPLE_PROCESS 0x1<<1    // * Tick() method is available    *
#define MVDSP_CANDO_WET_DRY_MIX    0x1<<2    // * Wet/Dry mix handled by module *
#define MVDSP_CANDO_AUTO_GAIN_MIX  0x1<<3    // * Auto-gain mix handled by mod  *
#define MVDSP_CANDO_PAD_PROCESS    0x1<<4    // * PadProcess() is available     *
#define MVDSP_CANDO_TEMPO_SYNC     0x1<<5    // * Module has UpdateBpm() method *

///////////////////////////////////////////////////////////////////////////////

// --- The MixVibes DSP-Module structure ---
typedef struct mixvibesDSPModule 
{
  // * parent window handle (set by host before Init() call) *
  HWND hwndParent;					

  // * instance handle of the plugin DLL (set by host before Init call) *
  HINSTANCE hDllInstance;			

  // * dsp version. on input: SDK version used to build this plugin  *
  //   on output (inside Init() call): SDK version supported by host *
  unsigned int mixvibesDspVersion;	
  
  // * plugin flags (see MVDSP_CANDO_XXX) *
  unsigned int mixvibesDspFlags;    

  unsigned int channels;       // * channel(s) used (mono or stereo, 1 or 2)    *
  unsigned int srate;          // * sampling rate holder (ex: 48000)            *

  float bpm;				   // * actual BPM (last value received)            *
  BOOL is_started;             // * flag used to track the DSP status (on/off)  *

  HBITMAP	hBitmap;           // * Bitmap displayed in the effect list in host *
                               // * may be null. set by plugin in Init()        *

  unsigned int nbParam;        // * number of parameters. set by plug in Init() *                        

  const mixvibesParam* tabParam; // * parameter definition structures array     *
  MVDSP_PARAM_VALUE* valNum;     // * parameter values in an array (set by plug)*      								

  void* puserData;             // * user specific data (ex: class instance ptr) *                     

  void (*Info)(HWND hwnd);     // * method to display information dialog        *

  // * Plugin initialization method. Called once *
  // (params: nbchan 1:mono 2:stereo, sampling rate, bpm of the media at start)
  struct mixvibesDSPModule*(*Init)(int sampling,int nbchannel,float bpm);  

  // * Called when DSP is activated. The start/def value of each param is given *
  void (*SetStart)(struct mixvibesDSPModule* this_mod,BOOL start,MVDSP_PARAM_VALUE* valNum);

  // * DSP-Block processing method. Where the computing takes place *
  int (*Process)(struct mixvibesDSPModule* this_mod, 
		float* samples, // * input sample array (stereo samples are interleaved)* 
		int numsamples, // * number of mono samples or sample pairs to process  *
		int position);	// * position in ms in the current media                *		 

  // * Process one mono sample or one stereo samples pair *
  void (*Tick)(struct mixvibesDSPModule* this_mod,float* samples);

  // * Called when unloading the plugin. Ressource deallocation here *
  void (*Quit)(struct mixvibesDSPModule* this_mod);    

  // * Called each time a parameter changes *
  void (*OnChange)(struct mixvibesDSPModule* this_mod, 	  
	  			   unsigned int indiceTab,    // * param indice *
			       MVDSP_PARAM_VALUE value);  // * new value    * 

  // * Retrieve parameter display string (stuffed by plugin) *
  void (*ParamDisplay)(struct mixvibesDSPModule* this_mod, int index, MVDSP_PARAM_VALUE val, char* disp_string);

  // * Called each time the bpm change in host *
  void (*UpdateBpm)(struct mixvibesDSPModule* this_mod,float bpm);

  // * Called when user has moved the pad *
  BOOL (*PadProcess)(struct mixvibesDSPModule* this_mod,float x,float y,float intensity);

} mixvibesDSPModule;

///////////////////////////////////////////////////////////////////////////////

// --- Header structure definition : static structure in main .dll source file ---
typedef struct 
{
  int version;                              // * header version            *
  char* description;                        // * description of the plugin *
  char* vendor;                             // * vendor / author string    *
  mixvibesDSPModule* (*getModule)(void);	// * module retrieval function *
} mixvibesDSPHeader;

  // Note : version must be set to MIXVIBES_DSP_SDK_VERSION

///////////////////////////////////////////////////////////////////////////////

// --- The only exported symbol of the .dll : allows to retrieve the header ---
typedef mixvibesDSPHeader* (*mixvibesDSPGetHeaderType)();

#endif